/*
Simple Clock
(C)2011 Bernardo Giovanni
www.settorezero.com

 vietata la ridistribuzione anche parziale
del seguente codice

il codice  realizzato per un pic16F627/628/648 con o senza A
le modifiche da apportare per altri pic sono indicate nell'articolo
*/

#include <htc.h>

// questa macro, _XTAL_FREQ,  usata dalle funzioni di ritardo integrate nel compilatore
#define	 _XTAL_FREQ 4000000

__CONFIG (FOSC_INTOSCIO & WDTE_OFF & PWRTE_OFF & MCLRE_OFF & BOREN_OFF & LVP_OFF & CPD_OFF & CP_OFF);

#define disp PORTA 	// Banco al quale sono collegati gli anodi dei display
#define H1   RB2 	// catodo display decine di ora
#define H2   RB3 	// catodo display unit di ora
#define M1 	 RB4 	// catodo display decine di minuti
#define M2 	 RB5 	// catodo display unit di minuti
#define dots RB1 	// puntini di separazione orario
#define bt1 RA5 // pulsante avanzamento minuti
#define bt2	RB0 // pulsante avanzamento ore

//Definizioni simboli per segmenti
#define	SA	128 //RA7
#define	SB	64	//RA6
#define	SC	16	//RA4
#define	SD	8
#define	SE	4
#define	SF	2
#define	SG	1 

#define N0 SA+SB+SC+SD+SE+SF
#define N1 SB+SC
#define N2 SA+SB+SG+SE+SD
#define N3 SA+SB+SG+SC+SD
#define N4 SF+SG+SB+SC
#define N5 SA+SF+SG+SC+SD
#define N6 SA+SF+SG+SE+SC+SD
#define N7 SA+SB+SC
#define N8 SA+SB+SF+SG+SC+SD+SE
#define N9 SA+SB+SF+SG+SC+SD

const unsigned char dispnum[]={N0,N1,N2,N3,N4,N5,N6,N7,N8,N9};
unsigned char hour=0, min=0, sec=0, halfsec=0; 
unsigned char muxcount=0; // contatore multiplex
unsigned char actdisplay=0; // flag che tiene conto del display correntemente visualizzato
unsigned char result=0;

void main(void)
	{
	TRISA=0b00100000; 
	PORTA=0b00000000;
	TRISB=0b11000001;
	PORTB=0b00000000;
	INTCON=0b01100000; //interrupt su timer0 e periferiche
	PIE1=0b00000001; // interrupt su timer1
	OPTION_REG=0b00000100; //prescaler a 1:32 ->interrupt ogni 5ms con preload a 100 da aumentare di 2 unit
	T1CON=0b00001111;
		
	// precarico timer0
	TMR0=102;
	// precarico timer1
	TMR1H=0xC0;
	TMR1L=0x00;
	dots=0; // puntini spenti
	GIE=1; // abilito gli interrupt
		
	while(1)
		{
		// pressione BT1 -> incremento ore
		if (!bt1)
			{
			__delay_ms(100);
			if (!bt1)
				{
				 hour++;
				 if(hour>23)
					{
					hour=0;
					}
				}
			}
		// pressione BT2 -> incremento minuti
		if (!bt2)
			{
			__delay_ms(100);
			if (!bt2)
				{
				 min++;
				 sec=0;
				 dots=1;
				 if(min>59)
					{
					min=0;
					}
				
				}
			}
					
		} // \WHILE(1)
	}// \MAIN


void interrupt ISR()
	{
	// interrupt su TIMER1
	// avviene ogni mezzo secondo
	if(TMR1IF)
		{
		TMR1H=0xC0; // ricarico solo la parte alta di timer1, la parte bassa  gi a zero e continua da sola
		dots^=1; // inverto lo stato dei puntini
		halfsec++; // mezzo secondo passato
				
		if (halfsec==2) // due mezzi secondi => un secondo
			{
			halfsec=0;
			sec++;
			}
		
		if (sec==60) //  passato un minuto
			{
			min++;
			sec=0;
			}
		
		if (min==60) //  passata un'ora
			{
			min=0;
			hour++;
			}
		
		if (hour==24) //  passata la mezzanotte
			{
			hour=0;
			}
		
		TMR1IF=0; // azzero il flag di interrupt
		}// \TMR1IF

	// interrupt su TIMER0
	// avviene ogni 5 millisecondi
	if(TMR0IF)
		{
		TMR0=102;
		disp=0; // azzero il display attualmente visualizzato per prevenire l'effetto "ghosting"
		// spengo tutti i catodi
		H1=0;
		H2=0;
		M1=0;
		M2=0;
			
		// controllo il display da visualizzare
		switch (actdisplay)
			{
			case 0: // decina ore
				result=hour/10;
				// non mostro la decina dell'orario se vale zero
				if (result)
					{
					H1=1;
					}
				break;
			
			case 1: // unit ore
				result=hour%10;
				H2=1;
				break;
				
			case 2: // decina minuti
				result=min/10;
				M1=1;
				break;
			
			case 3: // unit minuti
				result=min%10;
				M2=1;
				break;
			}
	
		/*
		Imposto il registro sul quale sono collegati i display.
		I segmenti da accendere in base al numero da visualizzare sono
		contenuti nell'array "dispnum".
		*/
		disp = (dispnum[result] & 0xFF); // devo lasciare inalterato lo stato di RA5
		
		// cambio display
		// ogni display rimane acceso in pratica 5millisecondi
		actdisplay++;
			
		if (actdisplay>3)
			{
			actdisplay=0;
			}
		TMR0IF=0; // azzero il flag di interrupt
		} // \TMR0IF
	}// \ISR